@@ -42,6 +42,7 @@ if T.TYPE_CHECKING:
     from ..environment import Environment
     from ..linkers import DynamicLinker
     from ..programs import ExternalProgram
+    from .compilers import CompileCheckMode
 
 
 class FortranCompiler(CLikeCompiler, Compiler):
@@ -71,7 +72,7 @@ class FortranCompiler(CLikeCompiler, Compiler):
         if binary_name.is_file():
             binary_name.unlink()
 
-        source_name.write_text('print *, "Fortran compilation is working."; end')
+        source_name.write_text('program main; print *, "Fortran compilation is working."; end program', encoding='utf-8')
 
         extra_flags: T.List[str] = []
         extra_flags += environment.coredata.get_external_args(self.for_machine, self.language)
@@ -210,11 +211,20 @@ class GnuFortranCompiler(GnuCompiler, FortranCompiler):
     def get_module_outdir_args(self, path: str) -> T.List[str]:
         return ['-J' + path]
 
-    def language_stdlib_only_link_flags(self) -> T.List[str]:
-        return ['-lgfortran', '-lm']
+    def language_stdlib_only_link_flags(self, env: 'Environment') -> T.List[str]:
+        # We need to apply the search prefix here, as these link arguments may
+        # be passed to a different compiler with a different set of default
+        # search paths, such as when using Clang for C/C++ and gfortran for
+        # fortran,
+        search_dir = self._get_search_dirs(env)
+        search_dirs: T.List[str] = []
+        if search_dir is not None:
+            for d in search_dir.split()[-1][len('libraries: ='):].split(':'):
+                search_dirs.append(f'-L{d}')
+        return search_dirs + ['-lgfortran', '-lm']
 
     def has_header(self, hname: str, prefix: str, env: 'Environment', *,
-                   extra_args: T.Optional[T.List[str]] = None,
+                   extra_args: T.Union[None, T.List[str], T.Callable[['CompileCheckMode'], T.List[str]]] = None,
                    dependencies: T.Optional[T.List['Dependency']] = None,
                    disable_cache: bool = False) -> T.Tuple[bool, bool]:
         '''
@@ -222,23 +232,32 @@ class GnuFortranCompiler(GnuCompiler, FortranCompiler):
         __has_include which breaks with GCC-Fortran 10:
         https://github.com/mesonbuild/meson/issues/7017
         '''
-        fargs = {'prefix': prefix, 'header': hname}
-        code = '{prefix}\n#include <{header}>'
-        return self.compiles(code.format(**fargs), env, extra_args=extra_args,
+        code = f'{prefix}\n#include <{hname}>'
+        return self.compiles(code, env, extra_args=extra_args,
                              dependencies=dependencies, mode='preprocess', disable_cache=disable_cache)
 
 
-class ElbrusFortranCompiler(GnuFortranCompiler, ElbrusCompiler):
+class ElbrusFortranCompiler(ElbrusCompiler, FortranCompiler):
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice, is_cross: bool,
                  info: 'MachineInfo', exe_wrapper: T.Optional['ExternalProgram'] = None,
                  defines: T.Optional[T.Dict[str, str]] = None,
                  linker: T.Optional['DynamicLinker'] = None,
                  full_version: T.Optional[str] = None):
-        GnuFortranCompiler.__init__(self, exelist, version, for_machine, is_cross,
-                                    info, exe_wrapper, defines=defines,
-                                    linker=linker, full_version=full_version)
+        FortranCompiler.__init__(self, exelist, version, for_machine, is_cross,
+                                 info, exe_wrapper, linker=linker, full_version=full_version)
         ElbrusCompiler.__init__(self)
 
+    def get_options(self) -> 'KeyedOptionDictType':
+        opts = FortranCompiler.get_options(self)
+        fortran_stds = ['f95', 'f2003', 'f2008', 'gnu', 'legacy', 'f2008ts']
+        key = OptionKey('std', machine=self.for_machine, lang=self.language)
+        opts[key].choices = ['none'] + fortran_stds
+        return opts
+
+    def get_module_outdir_args(self, path: str) -> T.List[str]:
+        return ['-J' + path]
+
+
 class G95FortranCompiler(FortranCompiler):
 
     LINKER_PREFIX = '-Wl,'
@@ -336,7 +355,8 @@ class IntelFortranCompiler(IntelGnuLikeCompiler, FortranCompiler):
     def get_preprocess_only_args(self) -> T.List[str]:
         return ['-cpp', '-EP']
 
-    def language_stdlib_only_link_flags(self) -> T.List[str]:
+    def language_stdlib_only_link_flags(self, env: 'Environment') -> T.List[str]:
+        # TODO: needs default search path added
         return ['-lifcore', '-limf']
 
     def get_dependency_gen_args(self, outtarget: str, outfile: str) -> T.List[str]:
@@ -420,7 +440,8 @@ class PGIFortranCompiler(PGICompiler, FortranCompiler):
                           '2': default_warn_args,
                           '3': default_warn_args + ['-Mdclchk']}
 
-    def language_stdlib_only_link_flags(self) -> T.List[str]:
+    def language_stdlib_only_link_flags(self, env: 'Environment') -> T.List[str]:
+        # TODO: needs default search path added
         return ['-lpgf90rtl', '-lpgf90', '-lpgf90_rpm1', '-lpgf902',
                 '-lpgf90rtl', '-lpgftnrtl', '-lrt']
 
@@ -461,8 +482,18 @@ class FlangFortranCompiler(ClangCompiler, FortranCompiler):
                           '2': default_warn_args,
                           '3': default_warn_args}
 
-    def language_stdlib_only_link_flags(self) -> T.List[str]:
-        return ['-lflang', '-lpgmath']
+    def language_stdlib_only_link_flags(self, env: 'Environment') -> T.List[str]:
+        # We need to apply the search prefix here, as these link arguments may
+        # be passed to a different compiler with a different set of default
+        # search paths, such as when using Clang for C/C++ and gfortran for
+        # fortran,
+        # XXX: Untested....
+        search_dir = self._get_search_dirs(env)
+        search_dirs: T.List[str] = []
+        if search_dir is not None:
+            for d in search_dir.split()[-1][len('libraries: ='):].split(':'):
+                search_dirs.append(f'-L{d}')
+        return search_dirs + ['-lflang', '-lpgmath']
 
 class Open64FortranCompiler(FortranCompiler):
 
@@ -494,12 +525,32 @@ class NAGFortranCompiler(FortranCompiler):
                                  is_cross, info, exe_wrapper, linker=linker,
                                  full_version=full_version)
         self.id = 'nagfor'
+        # Warnings are on by default; -w disables (by category):
+        self.warn_args = {
+            '0': ['-w=all'],
+            '1': [],
+            '2': [],
+            '3': [],
+        }
 
-    def get_warn_args(self, level: str) -> T.List[str]:
-        return []
+    def get_always_args(self) -> T.List[str]:
+        return self.get_nagfor_quiet(self.version)
 
     def get_module_outdir_args(self, path: str) -> T.List[str]:
         return ['-mdir', path]
 
+    @staticmethod
+    def get_nagfor_quiet(version: str) -> T.List[str]:
+        return ['-quiet'] if version_compare(version, '>=7100') else []
+
+    def get_pic_args(self) -> T.List[str]:
+        return ['-PIC']
+
+    def get_preprocess_only_args(self) -> T.List[str]:
+        return ['-fpp']
+
+    def get_std_exe_link_args(self) -> T.List[str]:
+        return self.get_always_args()
+
     def openmp_flags(self) -> T.List[str]:
         return ['-openmp']
